@@ -14,9 +14,6 @@ would appreciate credit if this program or parts of it are used.
 #include <errno.h>
 #include <ctype.h>	/* for isspace */
 #include <time.h>	/* for time(3) */
-#if 0
-#include <setjmp.h>
-#endif
 
 #include "expect_cf.h"
 
@@ -136,12 +133,6 @@ int cmdtype;
 
 static int i_read_errno;/* place to save errno, if i_read() == -1, so it
 			   doesn't get overwritten before we get to read it */
-#if 0
-static jmp_buf env;	/* for interruptable read() */
-			/* longjmp(env,1) times out the read */
-			/* longjmp(env,2) restarts the read */
-static int env_valid = FALSE;	/* whether we can longjmp or not */
-#endif
 
 #ifdef SIMPLE_EVENT
 static int alarm_fired;	/* if alarm occurs */
@@ -628,12 +619,12 @@ pattern:
 	    /* useful for debugging but not otherwise used */
 
 	    ec.pat = objv[i];
-	    Tcl_IncrRefCount(ec.pat);
+	    if (eg->duration == EXP_PERMANENT) Tcl_IncrRefCount(ec.pat);
 
 	    i++;
 	    if (i < objc) {
 		ec.body = objv[i];
-		Tcl_IncrRefCount(ec.body);
+		if (eg->duration == EXP_PERMANENT) Tcl_IncrRefCount(ec.body);
 	    } else {
 		ec.body = NULL;
 	    }
@@ -1545,8 +1536,60 @@ expParityStrip(obj,offsetBytes)
 }
 #endif /*OBSOLETE*/
 
-/* Strip UTF-encoded nulls from object, beginning at offset */
+/* This function is only used when debugging.  It checks when a string's
+   internal UTF is sane and whether an offset into the string appears to
+   be at a UTF boundary.
+*/
 static void
+expValid(obj,offset)
+     Tcl_Obj *obj;
+     int offset;
+{
+  char *s, *end;
+  int len;
+
+  s = Tcl_GetStringFromObj(obj,&len);
+
+  if (offset > len) {
+    printf("offset (%d) > length (%d)\n",offset,len);
+    fflush(stdout);
+    abort();
+  }
+
+  /* first test for null terminator */
+  end = s + len;
+  if (*end != '\0') {
+    printf("obj lacks null terminator\n");
+    fflush(stdout);
+    abort();
+  }
+
+  /* check for valid UTF sequence */
+  while (*s) {
+    Tcl_UniChar uc;
+
+    s += Tcl_UtfToUniChar(s,&uc);
+    if (s > end) {
+      printf("UTF out of sync with terminator\n");
+      fflush(stdout);
+      abort();
+    }
+  }
+  s += offset;
+  while (*s) {
+    Tcl_UniChar uc;
+
+    s += Tcl_UtfToUniChar(s,&uc);
+    if (s > end) {
+      printf("UTF from offset out of sync with terminator\n");
+      fflush(stdout);
+      abort();
+    }
+  }
+}
+
+/* Strip UTF-encoded nulls from object, beginning at offset */
+static int
 expNullStrip(obj,offsetBytes)
     Tcl_Obj *obj;
     int offsetBytes;
@@ -1554,6 +1597,7 @@ expNullStrip(obj,offsetBytes)
     char *src, *src2;
     char *dest;
     Tcl_UniChar uc;
+    int newsize;       /* size of obj after all nulls removed */
 
     src2 = src = dest = Tcl_GetString(obj) + offsetBytes;
 
@@ -1563,7 +1607,9 @@ expNullStrip(obj,offsetBytes)
 	    dest += Tcl_UniCharToUtf(uc,dest);
 	}
     }
-    Tcl_SetObjLength(obj,offsetBytes + (dest - src2));
+    newsize = offsetBytes + (dest - src2);
+    Tcl_SetObjLength(obj,newsize);
+    return newsize;
 }
 
 /* returns # of bytes read or (non-positive) error of form EXP_XXX */
@@ -1736,7 +1782,7 @@ int key;
 	 * such strings.  Doing it here lets them be sent to the screen, just
 	 * in case they are involved in formatting operations
 	 */
-	if (esPtr->rm_nulls) expNullStrip(esPtr->buffer,esPtr->printed);
+	if (esPtr->rm_nulls) size = expNullStrip(esPtr->buffer,esPtr->printed);
 	esPtr->printed = size; /* count'm even if not logging */
     }
     return(cc);
@@ -2727,9 +2773,9 @@ char **argv;
 
     if (argc == 0) {
 	if (Default) {
-	    value = exp_default_match_max;
+	  value = exp_default_rm_nulls;
 	} else {
-	    value = esPtr->rm_nulls;
+	  value = esPtr->rm_nulls;
 	}
 	sprintf(interp->result,"%d",value);
 	return(TCL_OK);
